/**
  Graphics simulator demo program:
  	You get a finely discretized ground grid, to apply some nonlinear distortions.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2012-01-19 (Public Domain)
*/
#include "physics/world.h" /* physics::library and physics::object */
#include "ogl/glsl.h"

/* Just include library bodies here, for easy linking */
#include "physics/world.cpp" 
#include "physics/config.cpp"
#include "ogl/glsl.cpp"


/* Called create a new simulation */
void physics_setup(physics::library &lib) {
	lib.background[0]=lib.background[1]=0.6;
	lib.background[2]=1.0; // light blue
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs normal alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	/* Load up a GLSL vertex and pixel shader */
	GLhandleARB prog=MAKE_PROGRAM_OBJECT3(
		/* GLSL Vertex shader section: */
		void main(void) {
			gl_Position=gl_ModelViewProjectionMatrix*gl_Vertex;
			
			/* Make everything a little redder than normal */
			color=mix(gl_Color,vec4(1.0,0.0,0.0,1.0),0.5);
		}
	,	
		/* GLSL shared vertex/fragment section: */
		varying vec4 color;
	,	
		/* GLSL fragment shader section: */
		void main(void) {
			gl_FragColor=color;
		}
	);
	glUseProgramObjectARB(prog);
	
	/* Draw opaque white ground plane */
	float groundz=-1.0;
	float groundsize=30;
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glVertex3f(-groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,+groundsize,groundz);
	glVertex3f(-groundsize,+groundsize,groundz);
	glEnd();
	
	/* Draw grid on top.  No depth test while drawing ground grid. */
	glDisable(GL_DEPTH_TEST);
	glColor4f(0.2,0.2,0.2,0.2);
	glEnable(GL_LINE_SMOOTH);
	glLineWidth(2.0);
	if (camera.z>=groundz) { /* no grid if you're underneath ground */
		glBegin(GL_LINES);
		float del=1.0; /* split lines into segments this long */
		for (float grid=-groundsize;grid<=+groundsize;grid+=1.0) {
			for (float g2=-groundsize;g2+del<=+groundsize;g2+=del) {
				glVertex3f(g2,grid,groundz);  /* along X axis */
				glVertex3f(g2+del,grid,groundz); 
				glVertex3f(grid,g2,groundz);  /* along Y axis */
				glVertex3f(grid,g2+del,groundz); 
			}
		}
		glEnd();
	}
	
	glUseProgramObjectARB(0); /* turn off our shader */
	
	/* Do depth while drawing objects */
	glEnable(GL_DEPTH_TEST);
	lib.world->draw(lib);
}


